@@ -407,6 +407,13 @@ def server_main(args=None):
     '''Run by the easycluster script.'''
     server_check_runner()
 
+    if sys.platform.startswith('win'):
+        from easycluster import windows_service as servmod
+    elif sys.platform.startswith('linux'):
+        from easycluster import linux_service as servmod
+    else:
+        servmod = None
+
     from optparse import OptionParser
     if args is None:
         args = sys.argv[1:]
@@ -417,11 +424,57 @@ def server_main(args=None):
     options.add_option('-S', '--serve', metavar='FILE', action='store_true', help='Run the server on the specified port')
     options.add_option('-g', '--generate', metavar='FILE', help='Generate random HMAC key and '
                        'write it to FILE. The file will be created with read access only for the owner.')
-    options.add_option('-O', '--overwrite', metavar='FILE', action='store_true', help='Allow --generate to overwrite existing keyfile')
+    options.add_option('-O', '--overwrite', action='store_true', help='Allow --generate to overwrite existing keyfile')
     options.add_option('-p', '--port', type='int', default=_core.DEFAULT_PORT, help='TCP port to listen on. Default: %d' % _core.DEFAULT_PORT)
     options.add_option('-c', '--class', dest='svrclass', default='Server', help='Fully-qualified name of server class')
+
+    if _daemonize:
+        options.add_option('-d', '--daemonize', action='store_true',
+                           help='Detach from current terminal and run in the background')
+        options.add_option("-P", "--pidfile", action="store",  
+                           dest="pidfile", metavar="FILE", 
+                           help="write daemon process ID to a file") 
+
+    if servmod:
+        options.add_option('-i', '--install', action='store_true', 
+                           help='Install EasyCluster as a service. The keyfile '
+                           'for this service is located at %s. If a keyfile is '
+                           'specified, it will be copied there; if no key exists '
+                           'currently, or if -O is given, a new key file will be '
+                           'written.' % servmod.KEY_PATH)  
+        options.add_option('-u', '--uninstall', action='store_true',
+                           help='Uninstall the service.')
+        options.add_option('--start', action='store_true', help='Start the service.')
+        options.add_option('--stop', action='store_true', help='Stop the service, but don\'t uninstall it.')
+
     opts, args = options.parse_args(args)
 
+    if servmod:
+        if opts.install:
+            key = _core.key_from_options(opts, False)
+            if key:
+                _core.write_key_file(servmod.KEY_PATH, key, True)
+            else:
+                _core.write_key_file(servmod.KEY_PATH, _core.generate_key(), opts.overwrite)
+
+            servmod.install_service()
+            if opts.start:
+                servmod.start_service()
+            return 0
+
+        if opts.uninstall:
+            servmod.stop_service()
+            servmod.uninstall_service()
+            return 0
+
+        if opts.start:
+            servmod.start_service()
+            return 0
+
+        if opts.stop:
+            servmod.stop_service()
+            return 0
+
     if opts.generate:
         if not _core.write_key_file(opts.generate, _core.generate_key(), opts.overwrite):
             print('Key file already exists (use --overwrite to overwrite it)', file=sys.stderr)
@@ -435,11 +488,13 @@ def server_main(args=None):
         else:
             clas = globals()[opts.svrclass]
 
-        run_server(opts.port, key, clas)
+        run_server(opts.port, key, clas, 
+                   getattr(opts, 'daemonize', False),
+                   getattr(opts, 'pidfile', None))
     else:
         options.print_help()
 
-def run_server(port, key, svrclass=Server):
+def run_server(port, key, svrclass=Server, daemonize=False, pidfile=None):
     '''Runs a server on the specified port.
 
     port is the TCP port to listen on.
@@ -470,8 +525,17 @@ def run_server(port, key, svrclass=Server):
     _server_socket = sock
 
     print('Listening on port %d' % port)
+
+    if daemonize:
+        _daemonize(pidfile)
+
     while _server_socket:
-        r, w, e = select.select([sock], (), (), 2)
+        try:
+            r, w, e = select.select([sock], (), (), 2)
+        except (socket.error, select.error) as e:
+            if e[0] not in _core._nonblocking_errors:
+                raise
+            
         if not r:
             continue
         try:
@@ -615,9 +679,8 @@ if sys.platform.startswith('win'):
             if fil.endswith('__main__.py'):
                 mainmod.__file__ = fil[:-11] + '__init__.py'
 
+    _daemonize = None
 
-    def install_service(key, user):
-        pass
 else:
     import signal
 
@@ -653,5 +716,29 @@ else:
     def server_init():
         signal.signal(signal.SIGCLD, _sigcld)
 
-    def install_service(key, user):
-        pass
+    def _daemonize(pidfile=None):
+        """Makes the process into a daemon."""
+        
+        cpid = os.fork()
+        if cpid != 0:
+            if pidfile is not None:
+                try:
+                    with open(pidfile, 'w') as fp:
+                        fp.write('%d\n' % cpid)
+                except Exception:
+                    os.kill(cpid, signal.SIGTERM)
+                    raise
+            os._exit(0)
+        
+        closefd = True
+        fd = os.open(os.devnull, os.O_RDWR)
+        for tfd in (0, 1, 2):
+            if tfd == fd:
+                closefd = False
+            else:
+                os.dup2(fd, tfd)
+        if closefd:
+            os.close(fd)
+
+        os.setsid()
+
